<?php
include_once 'fefunctions.php';
include_once 'iocommand.php';
start_session();
ini_set('display_errors', '0');
ini_set('error_reporting', E_NONE);

define('AI', $_SESSION['diagsDirectory'].'hpdiagsai');
$stdout = '';
$stderr = '';

$foundDevices = '(['; // This will be a JSON string

// If we are running online, we will include the hardrive location
if (isOnline())
{
  $mount = str_replace('\\', '/', $_SESSION['diagsDirectory']);
  $foundDevices .= '{';
  if (isWinOS())
    $foundDevices .= '"target":"winhardrive",'; // we use a different save method for windows HD
  else
    $foundDevices .= '"target":"hardrive",';
  $foundDevices .= '"label":"hardrive",';
  $foundDevices .= '"suffix":"'.$mount.'",';
  $foundDevices .= '"device":"",';
  $foundDevices .= '"mount":"'.$mount.'"';
  $foundDevices .= '},';
}
// Check for an internal floppy bay
$xml = new DomDocument($_SESSION['idResult']);
$xpath = xpath_new_context($xml->document_element());
$disketteDevice = xpath_eval($xpath, 'category/device[@class="Diskette"]');
if (count($disketteDevice->nodeset) > 0)
{
  if (isWinOS())
  {
    // For windows, check for a:/ as mount
    if (file_exists('a:/'))
    {
      $foundDevices .= '{';
      $foundDevices .= '"target":"winfloppy",'; // we use a different save method for windows FD
      $foundDevices .= '"label":"floppy",';
      $foundDevices .= '"suffix":"a:/",';
      $foundDevices .= '"device":"",';
      $foundDevices .= '"mount":"a:/"';
      $foundDevices .= '},';
    }
  }
  else
  {
    // For *nix, check for /dev/fd0 as device
    if (file_exists('/dev/fd0'))
    {
      $foundDevices .= '{';
      $foundDevices .= '"target":"floppy",';
      $foundDevices .= '"label":"floppy",';
      $foundDevices .= '"suffix":"",';
      $foundDevices .= '"device":"/dev/fd0",';
      $foundDevices .= '"mount":"/mnt/floppy"';
      $foundDevices .= '},';
    }
  }
}
// Check for connected USB Devices (*nix only)
if (!isWinOS())
{
  $usbFiles = glob('/proc/scsi/usb-storage*/*'); // Get all USB Storage files
  if (count($usbFiles) > 0)
  {
    $usbDevices = array();
    foreach($usbFiles as $file)
    {
      $fileContent = file_get_contents($file);
      $matches = array();
      $pattern = "/Host[\s]+([^\s:]+):.*Protocol:[\s]+(.*)Transport:.*/";
      $match = preg_match($pattern, clean($fileContent), $matches); // Parse the file for Host and Protocol
      // Continue gathering information only if we were able to extract what we needed
      if ($match > 0)
      {
        $usbDevices[] = array();
        $usbDevice = &$usbDevices[count($usbDevices)-1]; // reference last element of $usbDevices array
        $usbDevice['host'] = trim($matches[1]);
        $usbDevice['protocol'] = trim($matches[2]);
        // Some distros (RH3) specify an Attached value. If Attached = Yes or Attached is missing, then device is currently connected
        $pattern = "/Attached:[\s]+([^\s]*)/";
        $match = preg_match($pattern, clean($fileContent), $matches);
        if ($match > 0)
        {
          if (strstr($matches[1], 'Yes'))
            $usbDevice['attached'] = true;
          else
            $usbDevice['attached'] = false;
        }
        else
        {
          $usbDevice['attached'] = true;
        }
        // Only continue if device is attached
        if ($usbDevice['attached'] == true) 
        {
          // Now we will read the output of sg_scan and sg_map to locate the correct device
          $command = '<systemCall>/opt/hp/hpdiags/sg_scan</systemCall>';
          ExecuteIOCommand(AI, $command, $stdout, $stderr);
          $matches = array();
          $pattern = '/(\/dev\/[^\s]+):[\s]+'.$usbDevice['host'].'/';
          $match = preg_match($pattern, extractText($stdout), $matches);
          // Only continue if we read sg_scan properly
          if ($match > 0)
          {
            $devsg = $matches[1]; // devsg is the middle man between the scsi host and the device
            $command = '<systemCall>/opt/hp/hpdiags/sg_map</systemCall>';
            ExecuteIOCommand(AI, $command, $stdout, $stderr);
            $matches = array();
            $pattern = '/'.escape($devsg).'[\s]+([^\s]+)[\s]+/';
            $match = preg_match($pattern, extractText($stdout), $matches);
            // Only continue if we read sg_map properly
            if ($match > 0)
            {
              $usbDevice['device'] = $matches[1];
              // Compare device to its entry in /proc/scsi/scsi to determine if it is a R/W device
              if (file_exists('/proc/scsi/scsi'))
              {
                $fileContents = file_get_contents('/proc/scsi/scsi');
                $matches = array();
                $pattern = '/Host:[\s]*'.escape($usbDevice['host']).'[\s]+.*?Vendor:[\s]*([^\s]+).*?Type:[\s]*([^\s]*)/';
                $match = preg_match($pattern, clean($fileContents), $matches);
                if ($match > 0)
                {
                  $usbDevice['vendor'] = trim($matches[1]);
                  $usbDevice['type'] = trim($matches[2]);
                }
                else
                {
                  // If we didn't read /proc/scsi/scsi properly, fill in values with blank strings
                  $usbDevice['vendor'] = '';
                  $usbDevice['type'] = '';
                }
              }
              // If /proc/scsi/scsi doesn't exist, we will continue anyway with fingers crossed
            }
            else
            {
              // If we didn't read sg_map properly, remove this listing
              array_pop($usbDevices);
            }
          }
          else
          {
            // If we didn't read sg_scan properly, remove this listing
            array_pop($usbDevices);
          }
        }
        else
        {
          // If device is not attached, remove this listing
          array_pop($usbDevices);
        }
      }
    }
    foreach($usbDevices as $device)
    {
      // Check each device and fill in missing information
      if ($device['protocol'] == 'Transparent SCSI' && $device['type'] == 'Direct-Access')
      {
        $device['target'] = 'diskonkey';
        $device['label'] = 'usb_key';
        $device['suffix'] = $device['vendor'];
        $device['mount'] = '/mnt/diskonkey';
      }
      else if (strstr($device['protocol'], 'Floppy'))
      {
        $device['target'] = 'usbfloppy';
        $device['label'] = 'usb_floppy';
        $device['suffix'] = '';
        $device['mount'] = '/mnt/usbfloppy';
      }
      else
      {
        // This device does not meet requirements for a valud USB R/W device, will ignore
        $device['target'] = 'unknown';
      }
      // Put all valid devices into $foundDevices
      if ($device['target'] != 'unknown' && $device['target'] != '')
      {
        $foundDevices .= '{';
        $foundDevices .= '"target":"'.$device['target'].'",';
        $foundDevices .= '"label":"'.$device['label'].'",';
        $foundDevices .= '"suffix":"'.$device['suffix'].'",';
        $foundDevices .= '"device":"'.$device['device'].'",';
        $foundDevices .= '"mount":"'.$device['mount'].'"';
        $foundDevices .= '},';
      }
    }
  }
}
$foundDevices .= '])';
exit($foundDevices); // outputs string and exits this script

// Utility functions

// Converts newlines to spaces and multiple spaces into a single space
function clean($string)
{
  $string = str_replace("\n", " ", $string);
  $string = preg_replace("/[\s]+/", " ", $string);
  return str_replace("\n", " ", $string);
}

// Extracts the stdout string from an AI systemCall XML response
function extractText($xml)
{
  $dom = new DomDocument($xml);
  $xpath = xpath_new_context($dom->document_element());
  $stdout = xpath_eval($xpath, "//stdout");
  $string =  $stdout->nodeset[0]->get_content();
  return clean($string);
}

// Escapes '/' characters for a regexp
function escape($string)
{
  $string = clean($string);
  return str_replace("/", "\\/", $string);
}

?>
